use crossterm::{
	event::{poll, read, Event, KeyCode},
	terminal, ExecutableCommand, cursor,
};
use std::panic;
use std::{
	io::{stdout, Result, Write},
	time::Duration,
};

fn main() -> Result<()> {
	// Setup panic handling
	let default_panic = panic::take_hook();
	panic::set_hook(Box::new(move |info| {
		let _ = restore();
		default_panic(info);
	}));

	// Start terminaling
	init()?;
	let mut stdout = stdout();
	print!("Press Q to quit");

	// Main loop
	let quitting = false;
	while !quitting {
		//stdout.execute(terminal::BeginSynchronizedUpdate)?;
		handle_events()?;
		stdout.flush()?;
		//stdout.execute(terminal::EndSynchronizedUpdate)?;
	}

	// Stop terminaling
	restore()?;
	Ok(())
}

fn init() -> Result<()> {
	terminal::enable_raw_mode()?;
	stdout().execute(terminal::EnterAlternateScreen)?;
	Ok(())
}

fn restore() -> Result<()> {
	stdout().execute(terminal::LeaveAlternateScreen)?;
	terminal::disable_raw_mode()?;
	Ok(())
}

fn handle_events() -> Result<()> {
	if poll(Duration::from_secs(0))? {
		match read()? {
			Event::Key(event) => match event.code {
				KeyCode::Char('q') => {
					panic!("Just quitting, in a way...");
				},
				KeyCode::Char('a') => {
					print!("█");
				},
				KeyCode::Left => {
					stdout().execute(cursor::MoveLeft(1))?;
				},
				KeyCode::Right => {
					stdout().execute(cursor::MoveRight(1))?;
				},
				KeyCode::Up => {
					stdout().execute(cursor::MoveUp(1))?;
				},
				KeyCode::Down => {
					stdout().execute(cursor::MoveDown(1))?;
				},
				_ => {}
			},
			_ => {}
		}
	}
	Ok(())
}
